<?php
/**
 * Variation functions and definitions
 *
 * @since 1.1.5
 */

add_action( 'after_setup_theme', 'ignition_medi_variation_underwood_setup' );
/**
 * Sets up theme defaults and registers support for various WordPress features.
 *
 * @since 1.1.5
 *
 * Note that this function is hooked on the init hook which is too late for some features.
 * The hook after_setup_theme can't be used at all, as this file isn't loaded until it fires.
 */
function ignition_medi_variation_underwood_setup() {
	$variation = ignition_medi_get_theme_variation();
	$suffix    = ignition_medi_ignition_scripts_styles_suffix();

	add_editor_style( "theme-variations/{$variation}/style{$suffix}.css" );

	add_theme_support( 'editor-color-palette', array(
		array(
			'name'  => __( 'Primary color', 'ignition-medi' ),
			'slug'  => 'theme-primary',
			'color' => '#152242',
		),
		array(
			'name'  => __( 'Secondary color', 'ignition-medi' ),
			'slug'  => 'theme-secondary',
			'color' => '#b41f34',
		),
		array(
			'name'  => __( 'Primary text', 'ignition-medi' ),
			'slug'  => 'theme-primary-text',
			'color' => '#4c5b68',
		),
		array(
			'name'  => __( 'Secondary text', 'ignition-medi' ),
			'slug'  => 'theme-secondary-text',
			'color' => '#708496',
		),
		array(
			'name'  => __( 'Border', 'ignition-medi' ),
			'slug'  => 'theme-border',
			'color' => '#dbdee1',
		),
	) );
}

add_action( 'wp', 'ignition_medi_variation_underwood_register_scripts' );
/**
 * Registers scripts and styles unconditionally.
 *
 * @since 1.1.5
 */
function ignition_medi_variation_underwood_register_scripts() {
	$suffix    = ignition_medi_ignition_scripts_styles_suffix();
	$variation = ignition_medi_get_theme_variation();

	wp_register_style( 'ignition-medi-variation-style', get_theme_file_uri( "theme-variations/{$variation}/style{$suffix}.css" ), array(), ignition_medi_asset_version() );

	wp_register_style( 'ignition-medi-variation-underwood-elemementor-styles', get_theme_file_uri( "theme-variations/{$variation}/inc/assets/css/elementor{$suffix}.css" ), array(), ignition_medi_asset_version() );
}

add_filter( 'ignition_medi_styles_after_main', 'ignition_medi_variation_underwood_styles_after_main' );
/**
 * Enqueues theme variation styles after the main theme's stylesheet.
 *
 * @since 1.1.5
 */
function ignition_medi_variation_underwood_styles_after_main( $styles_after ) {
	if ( defined( 'ELEMENTOR_VERSION' ) ) {
		$styles_after = array_merge( $styles_after, array( 'ignition-medi-variation-underwood-elemementor-styles' ) );
	}

	return array_merge( $styles_after, array( 'ignition-medi-variation-style' ) );
}

add_filter( 'ignition_customizer_defaults', 'ignition_medi_variation_underwood_filter_customizer_defaults', 20 );
/**
 * Modifies the customizer's default values.
 *
 * @since 1.1.5
 *
 * @param array $defaults
 *
 * @return array
 */
function ignition_medi_variation_underwood_filter_customizer_defaults( $defaults ) {
	$theme = wp_get_theme();

	// Font family values should match fonts.json 'family' field.
	$primary_font   = 'Roboto';
	$secondary_font = 'Roboto';

	$colors = array(
		'white'               => '#ffffff',
		'black'               => '#000000',
		'grey'                => '#f1f1f1',
		'green'               => '#4bbf39',
		'red'                 => '#d62a29',
		'yellow'              => '#ffa500',
		'info'                => '#70a8e4',

		'body-bg'             => '#ffffff',
		'accent-color'        => '#152242',
		'accent-color-alt'    => '#b41f34',
		'text-color'          => '#4c5b68',
		'text-color-light'    => '#708496',
		'text-on-accent'      => '#ffffff',
		'border-color'        => '#dbdee1',

		'button-bg-color'     => '#b41f34',
		'button-border-color' => '#bf1616',
	);

	// phpcs:disable WordPress.Arrays.MultipleStatementAlignment.DoubleArrowNotAligned
	$defaults = array_merge( $defaults, array(
		'site_layout_type' => 'content_sidebar',

		'site_colors_secondary'      => $colors['accent-color-alt'],
		'site_colors_primary'        => $colors['accent-color'],
		'site_colors_text'           => $colors['text-color'],
		'site_colors_secondary_text' => $colors['text-color-light'],
		'site_colors_heading'        => '#152242',
		'site_colors_border'         => $colors['border-color'],

		'site_colors_buttons_background' => $colors['button-bg-color'],
		'site_colors_buttons_border'     => $colors['button-border-color'],

		'site_typo_primary'      => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => $primary_font,
					'variant'    => 'regular',
					'size'       => 16,
					'lineHeight' => 1.615,
					'transform'  => 'none',
					'spacing'    => 0.0,
					'is_gfont'   => true,
				),
			)
		),
		// Font attributes (size, lineHeight, etc) are disabled for this control.
		'site_typo_secondary'    => ignition_typography_control_defaults_empty_breakpoints( array(
			'desktop' => array(
				'family'   => $secondary_font,
				'variant'  => '700',
				'is_gfont' => true,
			),
		) ),
		'site_typo_navigation'   => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => 'Roboto',
					'variant'    => '700',
					'size'       => 14,
					'lineHeight' => 1.2,
					'transform'  => 'none',
					'spacing'    => -0.01,
					'is_gfont'   => true,
				),
			)
		),

		// Font attributes (size, lineHeight, etc) are disabled for this control.
		'site_typo_page_title'   => ignition_typography_control_defaults_empty_breakpoints( array(
			'desktop' => array(
				'family'   => $secondary_font,
				'variant'  => '700',
				'is_gfont' => true,
			),
		) ),
		// Font family and variant are disabled for these controls.
		'site_typo_h1'           => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => '',
					'variant'    => '',
					'size'       => 36,
					'lineHeight' => 1.15,
					'transform'  => 'none',
					'spacing'    => 0.0,
					'is_gfont'   => false,
				),
			)
		),
		'site_typo_h2'           => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => '',
					'variant'    => '',
					'size'       => 32,
					'lineHeight' => 1.15,
					'transform'  => 'none',
					'spacing'    => 0.0,
					'is_gfont'   => false,
				),
			)
		),
		'site_typo_h3'           => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => '',
					'variant'    => '',
					'size'       => 28,
					'lineHeight' => 1.2,
					'transform'  => 'none',
					'spacing'    => 0.0,
					'is_gfont'   => false,
				),
			)
		),
		'site_typo_h4'           => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => '',
					'variant'    => '',
					'size'       => 24,
					'lineHeight' => 1.25,
					'transform'  => 'none',
					'spacing'    => 0.0,
					'is_gfont'   => false,
				),
			)
		),
		'site_typo_h5'           => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => '',
					'variant'    => '',
					'size'       => 20,
					'lineHeight' => 1.25,
					'transform'  => 'none',
					'spacing'    => 0.0,
					'is_gfont'   => false,
				),
			)
		),
		'site_typo_h6'           => ignition_typography_control_defaults_empty_breakpoints( array(
			'desktop' => array(
				'family'     => '', // Font family is disabled for this control.
				'variant'    => '', // Font variant is disabled for this control.
				'size'       => 16,
				'lineHeight' => 1.32,
				'transform'  => 'none',
				'spacing'    => 0.0,
				'is_gfont'   => false,
			),
		) ),
		'site_typo_widget_title' => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => '',
					'variant'    => '',
					'size'       => 18,
					'lineHeight' => 1.2,
					'transform'  => '',
					'spacing'    => 0.0,
					'is_gfont'   => false,
				),
			)
		),
		'site_typo_widget_text'  => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => '',
					'variant'    => '',
					'size'       => 15,
					'lineHeight' => 1.5,
					'transform'  => 'none',
					'spacing'    => 0.0,
					'is_gfont'   => false,
				),
			)
		),
		'site_typo_button'       => ignition_typography_control_defaults_empty_breakpoints(
			array(
				'desktop' => array(
					'family'     => 'Roboto',
					'variant'    => '700',
					'size'       => 14,
					'lineHeight' => 1.2,
					'transform'  => 'uppercase',
					'spacing'    => 0.0,
					'is_gfont'   => true,
				),
			)
		),

		'top_bar_layout_is_visible' => 1,

		'top_bar_colors_background' => $colors['accent-color'],
		'top_bar_colors_border'     => $colors['accent-color-alt'],

		'top_bar_transparent_colors_background' => 'rgba(21, 34, 66, 0)',
		'top_bar_transparent_colors_text'       => $colors['white'],
		'top_bar_transparent_colors_border'     => 'rgba(255, 255, 255, 0.1)',

		'header_layout_type'                    => 'normal',
		'header_layout_is_full_width'           => 0,
		'header_layout_menu_sticky_type'        => 'permanent',

		'header_colors_text'                     => '#132739',
		'header_colors_border'                   => 'rgba(180, 31, 52, 0)',
		'header_colors_submenu_background_hover' => $colors['accent-color-alt'],
		'header_colors_submenu_text'             => $colors['accent-color'],

		'header_transparent_colors_submenu_background_hover' => $colors['accent-color-alt'],

		'header_sticky_colors_submenu_background_hover' => $colors['accent-color-alt'],
		'header_sticky_colors_submenu_text'             => $colors['accent-color'],

		'header_mobile_nav_colors_background' => $colors['white'],
		'header_mobile_nav_colors_link'       => $colors['accent-color'],
		'header_mobile_nav_colors_border'     => 'rgba(0, 0, 0, 0.15)',

		'page_title_with_background_is_visible'            => 1,
		'page_title_with_background_height'                => ignition_customizer_defaults_empty_breakpoints( array(
			'desktop' => 350,
			'tablet'  => 350,
			'mobile'  => 350,
		) ),

		'page_title_colors_overlay'          => 'rgba(15,15,16,0.7)',

		'blog_archive_posts_layout_type'          => '1col',
		'blog_archive_excerpt_length'             => 45,
		'blog_archive_meta_comments_is_visible'   => 1,

		'blog_single_authorbox_is_visible'       => 0,
		'blog_single_comments_is_visible'        => 0,
		'blog_single_related_columns'            => '',

		'footer_colors_background'         => $colors['accent-color'],
		'footer_colors_border'             => 'rgba(255, 255, 255, 0.1)',

		'footer_credits_colors_background' => $colors['accent-color'],
		'footer_credits_colors_border'     => 'rgba(255, 255, 255, 0.1)',

		'site_identity_description_is_visible' => 0,
	) );
	// phpcs:enable

	return $defaults;
}

add_action( 'init', 'ignition_medi_variation_underwood_register_block_editor_block_styles', 15 );
/**
 * Registers custom block styles.
 *
 * @since 1.1.5
 */
function ignition_medi_variation_underwood_register_block_editor_block_styles() {
	register_block_style( 'gutenbee/post-types', array(
		'name'  => 'ignition-medi-variation-underwood-minimal-grid',
		'label' => __( 'Minimal grid', 'ignition-medi' ),
	) );

	register_block_style( 'gutenbee/image', array(
		'name'  => 'ignition-medi-variation-underwood-image-border',
		'label' => __( 'Theme border', 'ignition-medi' ),
	) );

	register_block_style( 'gutenbee/justified-gallery', array(
		'name'  => 'ignition-medi-variation-underwood-image-border',
		'label' => __( 'Theme border', 'ignition-medi' ),
	) );

	register_block_style( 'core/social-links', array(
		'name'  => 'ignition-medi-variation-underwood-social',
		'label' => __( 'Theme socials', 'ignition-medi' ),
	) );
}
